function [prob_inclusion,ESS,Beta,L,acceptance]=First_Order(Dataset,prior_intrcp,c1,c,N,P,thita,num_iter,num_burn,num_thin,varargin)

%FIRST_ORDER      This M-File calculates the the posterior gene inclusion probabilities and their effective sample size. The First-Order
%                 method described by Brooks et al. in the  article   "Efficient construction of reversible jump MCMC
%                 proposal distributions "( JRSS B (2003)) is used to explore efficiently the model
%                 and parameter space of the Bayesian  variable selection problem in probit
%                 regression with many covariates


%Last Modification: 11 November 2008
%%%%%%%%%%%%%%%%
%INPUT variables
%%%%%%%%%%%%%%%%
%Dataset:two matrix
%       n x 1 is the response variable
%       n x p contains the expession levels of all the genes.This matrix
%       has been centered.
%Prior_intrcp : character
%                'Proper' indicates to use a normal prior for the inercept.
%                'Improper' indicates to use the non-informative improper
%                prior for a location parameter with constant density.
%c1 : matrix 1 x 1
%   Prior variance of the constant term
%c : matrix 1 x 1
%   Prior variance of the regression coefficients.
%N : matrix 1 x 1
%  Maximum number of proposed new variables.
%P : matrix 1 x 1
%  The probability parameter of the binomial distribution used in the model proposal.
%thita : matrix 1 x 1
%      The prior mean of the model size.
%num_iter : matrix 1 x 1
%         Number of iteration of the MCMC.
%num_burn : matrix 1 x 1
%         Burn in period of the MCMC.
%num_thin : matrix 1 x 1
%         Thining of the MCMC.
%Optional Input:
%     numGenes : matrix 1 x 1.
%     The number of pre-selected genes to be included in the design matrix.
%
%%%%%%%%%%%%%%%%%
%OUTPUT variables
%%%%%%%%%%%%%%%%%
%prob_inclusion : matrix p x 1
%                 Posterior gene inclusion probabilities.
%ESS : matrix  p x 1
%    Effective sample size of the posterior gene inclusion probabilities.
%Beta : cell array 1 x num
%       Posterior sample of regression coefficients.
%L : cell array 1 x num
%    Posterior sample of inclusion variables.
%acceptance : matrix 1 x 1
%             Between model acceptance rate of the algorithm.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

target=Dataset(:,1);
X=Dataset;X(:,1)=[];

[n,p]=size(X);


%Pre-select the genes to be included usind the Dutoid(2002) method.
if (nargin==11 && p~=varargin{1})
    numGenes=varargin{1};
    numClass=max(target)+1;
    method1='Dudoit';
    newX=select_genes(X,target,numClass,numGenes,method1);
    X=newX.GENES;
    [n,p]=size(X);
end




%The probability that each explanatory variable is included in the model.
thita=thita/p;
count=sparse(p,p);

%Initial model
model=binornd(1,thita,1,p);
l=find(model);
k=length(l);
t=ones(n,1);
data=X(:,l);

%Model Probability
prob=k*log(thita)+(p-k)*log(1-thita);

betmodelac=0;

%Initial values of the auxiliary variable.
Y(find(target==1),1)=2;Y(find(target==0),1)=-2; % Initial value of Y.

%Log-likelihood of the model and number of predictors.
num=(num_iter-num_burn)/num_thin;
result=sparse(num,p);
%Initialize the cell arrays.
Beta=cell(1,num);
L=cell(1,num);


if isequal(prior_intrcp,'Proper')

    datamat=[t data];
    K=datamat'*datamat+diag([1/c1 1/c*ones(1,k)]);%Precision matrix.
    var=inv(K);%Covariance matrix.

else
    %For improper prior.
    K=data'*data+diag(1/c*ones(1,k));%Precision matrix.
    var=inv(K);%Covariance matrix.

end

warning off all

%Start the Reversible Jump McMc iteration.

for i=1:num_iter

    %Within model move.
    if isequal(prior_intrcp,'Proper')

        mu=var*datamat'*Y;
        beta=mu+chol(var)'*randn(k+1,1);


    else
        %For improper prior.
        mu=var*data'*Y;
        beta=mu+chol(var)'*randn(k,1);
        intrcp=mean(Y)+1/(sqrt(n))*randn;
        beta=[intrcp;beta];

    end

    %Impute the auxiliary variable.
    m=[t data]*beta;

    for j=1:n
        if target(j)==1
            Y(j)=test_truncnorm1(m(j),1,0,+inf);
        else
            Y(j)=test_truncnorm1(m(j),1,-inf,0);
        end
    end

    %Loglikelihood of the model.
    likmodel=loglikelihood(target,data,beta);

    %Between model move using Reversible Jump and First-Order method.
    %Propose new gamma.

    proposemodel=model;
    w=1+binornd(N,P); %Block size.
    bin_prob=binopdf(w-1,N,P);

    if k<=w
        move=2;
    else
        move=unidrnd(2);
    end


    if move==1
        %Deletion move
        %Propose model.
        test=0;

        while test~=w %Select w dinstinct predictors.
            r=unidrnd(k,1,w);
            test=length(unique(r));
        end

        proposemodel(l(r))=0;
        newl=find(proposemodel);
        newk=k-w;
        newprob=newk*log(thita)+(p-newk)*log(1-thita);
        newdata=X(:,newl);

        newbeta=beta;
        newbeta(1+r)=[]; %Propose newbeta.
        liknewmodel=loglikelihood(target,newdata,newbeta);%Log-likelihood of new model.

        %Probability to move from model to proposemodel.
        r12=1/(2*combination(k,w))*bin_prob;
        %Probability to move from proposemodel to model.
        if newk<=w
            r21=1/(combination(p-(k-w),w))*bin_prob;
        else
            r21=1/(combination(p-(k-w),w))*bin_prob;
        end

        f1=-(w/2)*log(c)+log(r12)-log(r21)+prob-newprob;
        z=[t newdata]*newbeta;
        incpr=X(:,l(r));

        Z1=target.*(normpdf(z)./normcdf(z));
        Z2=(1-target).*(normpdf(z)./normcdf(-z));
        f2=incpr'*Z1-incpr'*Z2;
        x=fsolve(@(x) solfirstorder(x,w,f1,f2),[zeros(w,1);1],optimset('Display','off','Jacobian','on'));

        mu=real(x(1:w));
        stdev=real(x(1+w));%Scale of the proposal.

        v=(beta(r+1)-mu)/stdev;

        accept=-(likmodel-liknewmodel-w/2*log(c)-(beta(r+1)'*beta(1+r))/(2*c)+(v'*v)/2+prob-newprob+log(r12)-log(r21)+w*log(stdev));

    else

        %Adittion move
        covariate=find(model==0);
        k1=length(covariate);
        test=0;

        while test~=w %Select w dinstinct predictors.
            r=unidrnd(k1,1,w);
            test=length(unique(r));
        end

        proposemodel(covariate(r))=1;

        newl=find(proposemodel);
        newk=k+w;
        newprob=newk*log(thita)+(p-newk)*log(1-thita);
        newdata=X(:,newl);

        %Probability to move from model to proposemodel.
        if k<=w
            r12 = 1/(combination(p-k,w))*bin_prob;

        else
            r12 = 1/(2*combination(p-k,w))*bin_prob;

        end

        %Probability to move from proposemodel to model.
        r21=1/(2*combination(k+w,w))*bin_prob;

        f1=-(w/2)*log(c)+log(r21)-log(r12)+newprob-prob;
        z=[t data]*beta;
        incpr=X(:,covariate(r));

        Z1=target.*(normpdf(z)./normcdf(z));
        Z2=(1-target).*(normpdf(z)./normcdf(-z));
        f2=incpr'*Z1-incpr'*Z2;
        x=fsolve(@(x) solfirstorder(x,w,f1,f2),[zeros(w,1);1],optimset('Display','off','Jacobian','on'));

        mu=real(x(1:w));
        stdev=real(x(1+w));%Scale of the proposal.

        u=randn(w,1);
        v=stdev*u+mu;

        newbeta=regcoef([l';covariate(r)'],[beta;v]);%New regression coefficients.

        liknewmodel=loglikelihood(target,newdata,newbeta);%Log-likelihood of new model.

        %Acceptance probability of between model move.
        accept=liknewmodel-likmodel-w/2*log(c)-v'*v/(2*c)+u'*u/2+newprob-prob+log(r21)-log(r12)+w*log(stdev);

    end

    accept=exp(accept);
    u=rand;

    if u<=accept



        beta=newbeta;

        model=proposemodel;
        data=newdata;
        prob=newprob;
        k=newk;
        l=newl;

        if isequal(prior_intrcp,'Proper')

            datamat=[t data];
            K=datamat'*datamat+diag([1/c1 1/c*ones(1,k)]);%Precision matrix.
            var=inv(K);%Covariance matrix.

        else
            %For improper prior.

            K=data'*data+diag(1/c*ones(1,k));%Precision matrix.
            var=inv(K);%Covariance matrix.

        end

        betmodelac=betmodelac+1;


    end

    if i<num_burn,continue,end

    if i==num_burn
        count=count+sparse(prod_var(l,k),[repmat(l,1,k)],ones(1,k*k),p,p);
        result((i-num_burn)/num_thin+1,l)=1;
        Beta{(i-num_burn)/num_thin+1}=beta';
        L{(i-num_burn)/num_thin+1}=l;



    elseif (mod(i,num_thin)==0)
        count=count+sparse(prod_var(l,k),[repmat(l,1,k)],ones(1,k*k),p,p);
        result((i-num_burn)/num_thin+1,l)=1;
        Beta{(i-num_burn)/num_thin+1}=beta';
        L{(i-num_burn)/num_thin+1}=l;



    end

end


prob_inclusion=full(diag(count))/num;

visited_pred=find(sum(count));
result=result(:,visited_pred);

[m_pos,M_pos,f_pos]=summary_integautocorr(result,'Positive');
ESS=num./f_pos';

acceptance=betmodelac/num_iter;
